package cn.palmte.work.controller.backend;

import cn.palmte.work.bean.*;
import cn.palmte.work.model.*;
import cn.palmte.work.pojo.ActHisTask;
import cn.palmte.work.service.ProjectBudgetService;
import cn.palmte.work.service.ProjectEstimateService;
import cn.palmte.work.service.ProjectFinalSevice;
import cn.palmte.work.service.ProjectService;
import cn.palmte.work.service.*;
import cn.palmte.work.utils.FreeMarkerUtil;
import cn.palmte.work.utils.InterfaceUtil;
import cn.palmte.work.utils.Utils;
import cn.palmte.work.utils.excel.ExportUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.history.HistoricProcessInstance;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.datetime.DateFormatter;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 项目管理功能
 *
 * @author xiongshiyan at 2021/10/29 , contact me with email yanshixiong@126.com or phone 15208384257
 */
@Controller
@RequestMapping("/project")
public class ProjectController extends BaseController {
    private static final Logger logger = LoggerFactory.getLogger(ProjectController.class);

    @Autowired
    private DeptRepository deptRepository;
    @Autowired
    private ProjectService projectService;
    @Autowired
    private ProjectEstimateService projectEstimateService;
    @Autowired
    private ProjectBudgetService projectBudgetService;
    @Autowired
    private ProjectTaskRecordService projectTaskRecordService;
    @Autowired
    private ActProcInsService actProcInsService;
    @Autowired
    private ProjectInstanceRelationRepository projectInstanceRelationRepository;
    @Autowired
    private ProjectFinalSevice projectFinalSevice;
    @Autowired
    private HistoryService historyService;
    @Autowired
    private ActTaskDefService actTaskDefService;
    @Autowired
    private ProjectSettleService projectSettleService;

    /**
     * 项目列表
     */
    @RequestMapping("/list")
    public String list(@RequestParam(value = "keywords", required = false) String keywords,
                       @RequestParam(value = PAGE_NUMBER, defaultValue = DEFAULT_PAGE_NUMBER) int pageNumber,
                       @RequestParam(value = PAGE_SIZE, defaultValue = DEFAULT_PAGE_SIZE) int pageSize,
                       Map<String, Object> model) {
        //当前登录人的角色类型
        model.put("keywords", keywords);
        model.put("adminId", InterfaceUtil.getAdminId());
        model.put("deptList", deptRepository.findAll());
        ConcurrentHashMap<String, String> searchInfo = getSearchInfo(keywords, model);
        model.put("pager", projectService.list(searchInfo, pageNumber, pageSize));
        /*Page<Project> myApproveProjects = projectService.findMyApproveProjects(searchInfo, InterfaceUtil.getAdminId(), 1, Integer.MAX_VALUE);
        model.put("hasApproveProjects", CollectionUtil.isNotEmpty(myApproveProjects.getList()) ? "1" : "0");*/
        return "admin/project_list";
    }

    /**
     * 项目导出
     */
    @RequestMapping("/export")
    public void export(@RequestParam(value = "keywords", required = false) String keywords, HttpServletResponse httpServletResponse) throws IOException {
        Map<String, String> searchInfo = getSearchInfo(keywords);
        downloadHeader(httpServletResponse, Utils.generateExcelName("项目报表"), "application/octet-stream");
        String[] headers = {"项目名称", "项目类型", "项目状态", "审核状态", "当前审核人", "项目创建者", "部门名称", "项目开始时间", "项目结束时间", "最后更新时间"};
        String[] exportColumns = {"name", "typeDesc", "statusDesc", "approveStatusDesc", "approveName", "creatorName", "deptName", "startDate", "endDate", "lastUpdateTime"};
        ExportUtils.exportToExcel(headers, exportColumns, 1, 10000,
                httpServletResponse.getOutputStream(), (pN, pS) -> projectService.list(searchInfo, pN, pS).getList());
    }


    /**
     * 新增，至项目概算
     */
    @RequestMapping("/add")
    public String add(Map<String, Object> model) {

        //TODO 根据登录人的部门
        Dept one = deptRepository.findOne(1);
        model.put("dept", one);

        return "admin/project_estimate_add";
    }

    /**
     * 新增概算项目 保存草稿
     */
    @RequestMapping("/estimateAddSave")
    public String estimateAddSave(Project project, EstimateBean estimateBean, Map<String, Object> model) {
        projectService.estimateAddSave(project, estimateBean, InterfaceUtil.getAdmin(), ApproveStatusEnum.APPROVAL_UNCOMMIT);
        return "redirect:/project/list";
    }

    /**
     * 新增概算项目 保存并提交审核
     */
    @RequestMapping("/estimateAddSaveAndApprove")
    public String estimateAddSaveAndApprove(Project project, EstimateBean estimateBean, Map<String, Object> model) throws Exception {
        projectService.estimateAddSaveAndApprove(project, estimateBean, InterfaceUtil.getAdmin());
        return "redirect:/project/list";
    }

    /**
     * 去编辑概算项目页面
     *
     * @param id
     * @param model
     * @return
     */
    @RequestMapping("/edit")
    public String edit(@RequestParam("id") int id, Map<String, Object> model) {
        Project project = projectService.getProject(id);
        model.put("project", project);
        EstimateBean estimateBean = projectEstimateService.getEstimate(project);
        model.put("estimateBean", estimateBean);
        //freemarker可以利用的静态方法
        model.put("Utils", FreeMarkerUtil.fromStaticPackage("cn.palmte.work.utils.Utils"));
        return "admin/project_estimate_edit";
    }

    /**
     * 编辑概算项目保存
     * @param project
     * @param estimateBean
     * @param model
     * @return
     */
    @RequestMapping("/estimateEditSave")
    public String estimateEditSave(Project project, EstimateBean estimateBean, Map<String, Object> model) {
        Project projectInDb = projectService.getProject(project.getId());
        projectService.estimateEditSave(project, projectInDb, estimateBean, InterfaceUtil.getAdmin(), ApproveStatusEnum.APPROVAL_UNCOMMIT);
        return "redirect:/project/list";
    }

    /**
     * 编辑概算项目保存并提交审批
     * @param project
     * @param estimateBean
     * @param model
     * @return
     * @throws Exception
     */
    @RequestMapping("/estimateEditSaveAndApprove")
    public String estimateEditSaveAndApprove(Project project, EstimateBean estimateBean, Map<String, Object> model) throws Exception {
        Project projectInDb = projectService.getProject(project.getId());
        projectService.estimateEditSaveAndApprove(project, projectInDb, estimateBean, InterfaceUtil.getAdmin());
        return "redirect:/project/list";
    }

    /**
     * 进项目预算页面
     */
    @RequestMapping("/budgetEdit")
    public String budget(@RequestParam("id") int id, Map<String, Object> model) {
        Project project = projectService.getProject(id);
        model.put("project", project);
        BudgetBean budgetBean = projectBudgetService.getBudget(project);
        //预算主页面数据
        model.put("budgetBean", budgetBean);
        //收入明细
        model.put("incomeDetails", projectBudgetService.getBudgetIncomeDetail(project));
        //成本明细
        model.put("costDetails", projectBudgetService.getBudgetCostDetail(project));
        //项目管理成本明细
        model.put("costProjectManageDetails", projectBudgetService.getBudgetCostProjectManageDetail(project));
        List<ProjectBudgetPlanDetail> projectBudgetPlanDetails = projectBudgetService.getProjectBudgetPlanDetails(project);
        //资金计划明细
        model.put("projectBudgetPlanDetails", projectBudgetPlanDetails);
        //资金计划总【上面汇总表】
        model.put("projectBudgetPlanDetailTotal", projectBudgetService.getProjectBudgetPlanDetailTotal(project, projectBudgetPlanDetails));
        //资金计划表中的统计信息【下面资金小表】
        model.put("underwrittenPlanStatistic", projectBudgetService.getProjectUnderwrittenPlanStatisticBean(projectBudgetPlanDetails));
        //现金表
        model.put("cashFlowBean", projectBudgetService.getCashFlowBean(project, projectBudgetPlanDetails));
        //freemarker可以利用的静态方法
        model.put("Utils", FreeMarkerUtil.fromStaticPackage("cn.palmte.work.utils.Utils"));
        return "admin/project_budget_edit";
    }

    /**
     * 进行项目预算保存
     */
    @RequestMapping("/budgetEditSave")
    public String budgetEditSave(Project project, BudgetBean budgetBean, Map<String, Object> model) {
        Project projectInDb = projectService.getProject(project.getId());
        projectService.budgetEditSave(project, projectInDb, budgetBean, InterfaceUtil.getAdmin(), ApproveStatusEnum.APPROVAL_UNCOMMIT);
        return "redirect:/project/list";
    }

    /**
     * 进行项目预算保存并提交审核
     */
    @RequestMapping("/budgetEditSaveAndApprove")
    public String budgetEditSaveAndApprove(Project project, BudgetBean budgetBean, Map<String, Object> model) throws Exception {
        Project projectInDb = projectService.getProject(project.getId());
        projectService.budgetEditSaveAndApprove(project, projectInDb, budgetBean, InterfaceUtil.getAdmin());
        return "redirect:/project/list";
    }

    /**
     * 保存收入明细
     */
    @RequestMapping("/budgetEditSaveIncomeDetail")
    @ResponseBody
    public ResponseMsg budgetEditSaveIncomeDetail(@RequestBody String body) {
        JSONObject jsonObject = JSON.parseObject(body);
        JSONArray details = jsonObject.getJSONArray("details");
        List<ProjectBudgetIncomeDetail> detailList = new ArrayList<>(details.size());
        for (int i = 0; i < details.size(); i++) {
            ProjectBudgetIncomeDetail detail = details.getObject(i, ProjectBudgetIncomeDetail.class);
            detailList.add(detail);
        }

        Project project = projectService.getProject(jsonObject.getInteger("projectId"));

        projectBudgetService.saveBudgetIncomeDetail(project, detailList);

        return ResponseMsg.buildSuccessMsg("成功");
    }

    /**
     * 保存成本明细
     */
    @RequestMapping("/budgetEditSaveCostDetail")
    @ResponseBody
    public ResponseMsg budgetEditSaveCostDetail(@RequestBody String body) {
        JSONObject jsonObject = JSON.parseObject(body);
        JSONArray details = jsonObject.getJSONArray("details");
        List<ProjectBudgetCostDetail> detailList = new ArrayList<>(details.size());
        for (int i = 0; i < details.size(); i++) {
            ProjectBudgetCostDetail detail = details.getObject(i, ProjectBudgetCostDetail.class);
            detailList.add(detail);
        }

        Project project = projectService.getProject(jsonObject.getInteger("projectId"));

        projectBudgetService.saveBudgetCostDetail(project, detailList);

        return ResponseMsg.buildSuccessMsg("成功");
    }

    /**
     * 保存项目管理成本明细
     */
    @RequestMapping("/budgetEditSaveCostProjectManageDetail")
    @ResponseBody
    public ResponseMsg budgetEditSaveCostProjectManageDetail(@RequestBody String body) {
        JSONObject jsonObject = JSON.parseObject(body);
        JSONArray details = jsonObject.getJSONArray("details");
        List<ProjectBudgetCostProjectManageDetail> detailList = new ArrayList<>(details.size());
        for (int i = 0; i < details.size(); i++) {
            ProjectBudgetCostProjectManageDetail detail = details.getObject(i, ProjectBudgetCostProjectManageDetail.class);
            detailList.add(detail);
        }

        Project project = projectService.getProject(jsonObject.getInteger("projectId"));

        projectBudgetService.saveBudgetCostProjectManageDetail(project, detailList);

        return ResponseMsg.buildSuccessMsg("成功");
    }

    /**
     * 保存项资金计划明细
     */
    @RequestMapping("/budgetEditSaveBudgetPlanDetail")
    @ResponseBody
    public ResponseMsg budgetEditSaveBudgetPlanDetail(@RequestBody String body) {
        JSONObject jsonObject = JSON.parseObject(body);
        JSONArray details = jsonObject.getJSONArray("details");
        List<ProjectBudgetPlanDetail> detailList = new ArrayList<>(details.size());
        for (int i = 0; i < details.size(); i++) {
            ProjectBudgetPlanDetail detail = details.getObject(i, ProjectBudgetPlanDetail.class);
            detailList.add(detail);
        }

        Project project = projectService.getProject(jsonObject.getInteger("projectId"));

        projectBudgetService.saveBudgetPlanDetail(project, detailList);

        return ResponseMsg.buildSuccessMsg("成功");
    }


    /**
     * 待我审核项目列表
     */
    @RequestMapping("/listApprove")
    public String listApprove(@RequestParam(value = "keywords", required = false) String keywords,
                              @RequestParam(value = PAGE_NUMBER, defaultValue = DEFAULT_PAGE_NUMBER) int pageNumber,
                              @RequestParam(value = PAGE_SIZE, defaultValue = DEFAULT_PAGE_SIZE) int pageSize,
                              Map<String, Object> model) {
        //当前登录人的角色类型
        model.put("keywords", keywords);
        model.put("deptList", deptRepository.findAll());
        ConcurrentHashMap<String, String> searchInfo = getSearchInfo(keywords, model);
        model.put("pager", projectService.findMyApproveProjects(searchInfo, InterfaceUtil.getAdminId(), pageNumber, pageSize));
        return "admin/project_list_approve";
    }

    /**
     * 待我审核项目导出
     */
    @RequestMapping("/exportApprove")
    public void exportApprove(@RequestParam(value = "keywords", required = false) String keywords, HttpServletResponse httpServletResponse) throws IOException {
        Map<String, String> searchInfo = getSearchInfo(keywords);
        downloadHeader(httpServletResponse, Utils.generateExcelName("待我审核项目报表"), "application/octet-stream");
        String[] headers = {"项目名称", "项目类型", "项目状态", "审核状态", "当前审核人", "项目创建者", "部门名称", "项目开始时间", "项目结束时间", "最后更新时间"};
        String[] exportColumns = {"name", "typeDesc", "statusDesc", "approveStatusDesc", "approveName", "creatorName", "deptName", "startDate", "endDate", "lastUpdateTime"};
        ExportUtils.exportToExcel(headers, exportColumns, 1, 10000,
                httpServletResponse.getOutputStream(), (pN, pS) -> projectService.findMyApproveProjects(searchInfo, InterfaceUtil.getAdminId(), pN, pS).getList());
    }


    /**
     * 项目查看页面
     *
     * @param id
     * @param model
     * @return
     */
    @RequestMapping("/detail")
    public String detail(@RequestParam("id") int id, Map<String, Object> model) {
        Project project = projectService.getProject(id);
        String time = "2021-11";
        EstimateBean estimateBean = projectEstimateService.getEstimate(project);
        model.put("estimateBean", estimateBean);
        model.put("project", project);
        model.put("formerBean", projectSettleService.getFormerSettle(project, time));
        BudgetBean budgetBean = projectBudgetService.getBudget(project);
        //预算主页面数据
        model.put("budgetBean", budgetBean);
        //收入明细
        model.put("incomeDetails", projectBudgetService.getBudgetIncomeDetail(project));
        //成本明细
        model.put("costDetails", projectBudgetService.getBudgetCostDetail(project));
        //项目管理成本明细
        model.put("costProjectManageDetails", projectBudgetService.getBudgetCostProjectManageDetail(project));
        List<ProjectBudgetPlanDetail> projectBudgetPlanDetails = projectBudgetService.getProjectBudgetPlanDetails(project);
        //资金计划明细
        model.put("projectBudgetPlanDetails", projectBudgetPlanDetails);
        //资金计划总【上面汇总表】
        model.put("projectBudgetPlanDetailTotal", projectBudgetService.getProjectBudgetPlanDetailTotal(project, projectBudgetPlanDetails));
        //资金计划表中的统计信息【下面资金小表】
        model.put("underwrittenPlanStatistic", projectBudgetService.getProjectUnderwrittenPlanStatisticBean(projectBudgetPlanDetails));
        //现金表
        model.put("cashFlowBean", projectBudgetService.getCashFlowBean(project, projectBudgetPlanDetails));
        model.put("finalBean", projectFinalSevice.getFinal(project));
        //freemarker可以利用的静态方法
        model.put("Utils", FreeMarkerUtil.fromStaticPackage("cn.palmte.work.utils.Utils"));
        List<ProjectTaskRecord> list = projectTaskRecordService.list(id);
        model.put("taskRecords", list);
        return "admin/project_detail";
    }

    /**
     * 审核页面
     * @param id
     * @param model
     * @return
     */
    @RequestMapping("/approve")
    public String approve(@RequestParam("id") int id, @RequestParam String listFrom, Map<String, Object> model) {
        String time = "2021-11";
        Project project = projectService.getProject(id);
        EstimateBean estimateBean = projectEstimateService.getEstimate(project);
        model.put("estimateBean", estimateBean);
        model.put("project", project);
        model.put("formerBean", projectSettleService.getFormerSettle(project, time));
        model.put("listFrom", listFrom);
        BudgetBean budgetBean = projectBudgetService.getBudget(project);
        //预算主页面数据
        model.put("budgetBean", budgetBean);
        //收入明细
        model.put("incomeDetails", projectBudgetService.getBudgetIncomeDetail(project));
        //成本明细
        model.put("costDetails", projectBudgetService.getBudgetCostDetail(project));
        //项目管理成本明细
        model.put("costProjectManageDetails", projectBudgetService.getBudgetCostProjectManageDetail(project));
        List<ProjectBudgetPlanDetail> projectBudgetPlanDetails = projectBudgetService.getProjectBudgetPlanDetails(project);
        //资金计划明细
        model.put("projectBudgetPlanDetails", projectBudgetPlanDetails);
        //资金计划总【上面汇总表】
        model.put("projectBudgetPlanDetailTotal", projectBudgetService.getProjectBudgetPlanDetailTotal(project, projectBudgetPlanDetails));
        //资金计划表中的统计信息【下面资金小表】
        model.put("underwrittenPlanStatistic", projectBudgetService.getProjectUnderwrittenPlanStatisticBean(projectBudgetPlanDetails));
        //现金表
        model.put("cashFlowBean", projectBudgetService.getCashFlowBean(project, projectBudgetPlanDetails));
        model.put("finalBean", projectFinalSevice.getFinal(project));
        //freemarker可以利用的静态方法
        model.put("Utils", FreeMarkerUtil.fromStaticPackage("cn.palmte.work.utils.Utils"));
        return "admin/project_approve";
    }

    /**
     * 审批完成任务
     * @param projectId
     * @param json
     * @return
     */
    @ResponseBody
    @RequestMapping("/completeTask/{projectId}")
    public ResponseMsg completeTask(@PathVariable int projectId, @RequestBody String json) {
        return projectService.completeTask(projectId, json);
    }


    /**
     * 查看审核流程
     * @param projectId
     * @param model
     * @return
     */
    @RequestMapping("/taskRecords/{projectId}")
    public String taskRecords(@PathVariable int projectId, Map<String, Object> model) {
        model.put("projectId", projectId);
        List<ProjectTaskRecord> list = projectTaskRecordService.list(projectId);
        if (list != null && !list.isEmpty()) {
            model.put("list", list);
        }

        List<ProjectInstanceRelation> relationList = projectInstanceRelationRepository.findByProjectIdOrderByCreateTimeDesc(projectId);
        if (relationList != null && !relationList.isEmpty()) {
            String procInsId = relationList.get(0).getProcessInsId();

            List<ActHisTask> hisTaskList = actTaskDefService.hisTaskList(procInsId);
            model.put("taskList", hisTaskList);

            HistoricProcessInstance historicProcessInstance = historyService.createHistoricProcessInstanceQuery().processInstanceId(procInsId).singleResult();
            if (historicProcessInstance != null) {
                model.put("procDefName", historicProcessInstance.getProcessDefinitionName());
            }
        }

        return "admin/project_task_record_list";
    }

    /**
     * 流程实列图片
     *
     * @param response
     * @param projectId
     * @throws Exception
     */
    @RequestMapping("/procInsPng/{projectId}")
    public void png(HttpServletResponse response, @PathVariable("projectId") int projectId) throws Exception {
        List<ProjectInstanceRelation> relationList = projectInstanceRelationRepository.findByProjectIdOrderByCreateTimeDesc(projectId);
        if (relationList == null || relationList.isEmpty()) {
            return;
        }
        actProcInsService.createProcInsPng(response, relationList.get(0).getProcessInsId());
    }

    @InitBinder
    public void initBinder(WebDataBinder webDataBinder) {
        webDataBinder.addCustomFormatter(new DateFormatter("yyyy-MM-dd"));
    }
}
